/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

#include "dirtree.h"

#include "pixmaps/refresh.xpm"
#include "pixmaps/up.xpm"
#include "pixmaps/down.xpm"
#include "pixmaps/left.xpm"
#include "pixmaps/right.xpm"
#include "pixmaps/dotfile.xpm"

#define min(a, b) (((a)<(b))?(a):(b))

DirView *dirview = NULL;

  /*
   * callback functions 
   */
static gint cb_dirview_button_press_event (GtkWidget * widget,
					   GdkEventButton * event,
					   gpointer data);

static gint cb_dirview_button_release_event (GtkWidget * widget,
					     GdkEventButton * event,
					     DirView * dv);
static gboolean cb_dirview_key_press (GtkWidget * widget, GdkEventKey * event,
				      DirView * dv);
static void cb_dirview_select_file (GtkWidget * widget, gchar * path,
				    DirView * dv);

static void cb_dirview_refresh_dir_tree (GtkWidget * widget, DirView * dv);
static void cb_dirview_expand (GtkWidget * widget, DirView * dv);
static void cb_dirview_show_hidden (GtkWidget * widget, DirView * dv);
static void cb_dirview_collapse (GtkWidget * widget, DirView * dv);
static void cb_dirview_down (GtkWidget * widget, DirView * dv);
static void cb_dirview_up (GtkWidget * widget, DirView * dv);

static GtkWidget *dirview_create_toolbar (DirView * dv);
static void dirview_update_toolbar (DirView * dv);

static GtkItemFactoryEntry dirview_popupmenu_factory[] = {
    {"/_Refresh       ", NULL, cb_dirview_refresh_dir_tree, 0, NULL}
};

static int dirview_popupmenu_factory_count =
    sizeof (dirview_popupmenu_factory) / sizeof (GtkItemFactoryEntry);

static GtkItemFactory *dirview_popup_item_factory;

void
dirview_create (const gchar * start_path, GtkWidget * parent_win)
{
    dirview = g_new0 (DirView, 1);

    /*
     * main vbox 
     */
    dirview->container = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (dirview->container);
    gtk_object_set_data_full (GTK_OBJECT (parent_win), "dirview_container",
			      dirview->container,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (dirview->container);

    /*
     * toolbar 
     */
    dirview->toolbar_eventbox = gtk_event_box_new ();
    gtk_widget_ref (dirview->toolbar_eventbox);
    gtk_object_set_data_full (GTK_OBJECT (parent_win),
			      "dirview_toolbar_eventbox",
			      dirview->toolbar_eventbox,
			      (GtkDestroyNotify) gtk_widget_unref);

    gtk_container_set_border_width (GTK_CONTAINER (dirview->toolbar_eventbox),
				    1);
    gtk_box_pack_start (GTK_BOX (dirview->container),
			dirview->toolbar_eventbox, FALSE, FALSE, 0);
    gtk_widget_show (dirview->toolbar_eventbox);

    dirview->toolbar = dirview_create_toolbar (dirview);
    gtk_container_add (GTK_CONTAINER (dirview->toolbar_eventbox),
		       dirview->toolbar);

    /*
     * scrolled window 
     */
    dirview->scroll_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_ref (dirview->scroll_win);
    gtk_object_set_data_full (GTK_OBJECT (parent_win), "dirview_scroll_win",
			      dirview->scroll_win,
			      (GtkDestroyNotify) gtk_widget_unref);

    gtk_container_set_border_width (GTK_CONTAINER (dirview->scroll_win), 1);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dirview->scroll_win),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start (GTK_BOX (dirview->container), dirview->scroll_win,
			TRUE, TRUE, 0);
    gtk_widget_show (dirview->scroll_win);

    /*
     * dirtree 
     */
    dirview->dirtree =
	dirtree_new (parent_win, start_path, conf.scan_dir, conf.check_hlinks,
		     conf.show_hidden, conf.dirtree_line_style,
		     conf.dirtree_expander_style);

    gtk_container_add (GTK_CONTAINER (dirview->scroll_win), dirview->dirtree);
    gtk_widget_set_usize (GTK_WIDGET (dirview->scroll_win),
			  conf.dirtree_width, conf.dirtree_height);
    gtk_widget_show (dirview->dirtree);

    /*
     * callback singnals 
     */
    gtk_signal_connect (GTK_OBJECT (dirview->dirtree), "button_press_event",
			GTK_SIGNAL_FUNC (cb_dirview_button_press_event),
			dirview);

    gtk_signal_connect (GTK_OBJECT (dirview->dirtree), "button_release_event",
			GTK_SIGNAL_FUNC (cb_dirview_button_release_event),
			dirview);

    gtk_signal_connect (GTK_OBJECT (dirview->dirtree), "key_press_event",
			GTK_SIGNAL_FUNC (cb_dirview_key_press), dirview);

    gtk_signal_connect (GTK_OBJECT (dirview->dirtree), "select_file",
			GTK_SIGNAL_FUNC (cb_dirview_select_file), dirview);

    gtk_widget_show (dirview->scroll_win);
    dirview_update_toolbar (dirview);
}

void
dirview_scroll_center (void)
{
    GtkAdjustment *vadj;
    gfloat  pos;

    vadj =
	gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW
					     (dirview->scroll_win));

    pos = (vadj->upper * GTK_CLIST (DIRTREE (dirview->dirtree))->focus_row)
	/ (gfloat) GTK_CLIST (dirview->dirtree)->rows;

    gtk_adjustment_set_value (vadj, min (vadj->upper - vadj->page_size, pos));
}

void
dirview_destroy (void)
{
    gtk_widget_destroy (dirview->dirtree);

    g_free (dirview);
    dirview = NULL;
}

 /*
  * local functions 
  */

static  gint
cb_dirview_button_press_event (GtkWidget * widget, GdkEventButton * event,
			       gpointer data)
{
    GtkWidget *dirview_popup;

    if (event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
	/*
	 * create popup menu 
	 */
	dirview_popup_item_factory =
	    gtk_item_factory_new (GTK_TYPE_MENU, "<DVPop>", NULL);
	gtk_item_factory_create_items (dirview_popup_item_factory,
				       dirview_popupmenu_factory_count,
				       dirview_popupmenu_factory, NULL);

	dirview_popup =
	    gtk_item_factory_get_widget (dirview_popup_item_factory,
					 "<DVPop>");

	/*
	 * popup menu 
	 */
	gtk_menu_popup (GTK_MENU (dirview_popup), NULL, NULL, NULL, NULL, 0,
			0);

#ifdef USE_GTK2
	gtk_object_ref (GTK_OBJECT (dirview_popup));
	gtk_object_sink (GTK_OBJECT (dirview_popup));
#endif

	gtk_widget_unref (dirview_popup);
    }

    return FALSE;
}

static  gint
cb_dirview_button_release_event (GtkWidget * widget, GdkEventButton * event,
				 DirView * dv)
{
    dirview_update_toolbar (dv);

    return FALSE;
}

static  gboolean
cb_dirview_key_press (GtkWidget * widget, GdkEventKey * event, DirView * dv)
{
    guint   keyval;
    GtkCTreeNode *node;
    GList  *sel_list;

    keyval = event->keyval;

    sel_list = GTK_CLIST (dv->dirtree)->selection;

    if (!sel_list)
	return FALSE;

    node = sel_list->data;

    switch (keyval)
    {
      case GDK_space:
	  dirtree_freeze (GTK_CTREE (dv->dirtree));
	  gtk_ctree_toggle_expansion (GTK_CTREE (dv->dirtree), node);
	  dirtree_thaw (GTK_CTREE (dv->dirtree));

	  break;
      case GDK_Right:
	  dirtree_freeze (GTK_CTREE (dv->dirtree));
	  gtk_ctree_expand (GTK_CTREE (dv->dirtree), node);
	  dirtree_thaw (GTK_CTREE (dv->dirtree));

	  break;
      case GDK_Left:
	  dirtree_freeze (GTK_CTREE (dv->dirtree));
	  gtk_ctree_collapse (GTK_CTREE (dv->dirtree), node);
	  dirtree_thaw (GTK_CTREE (dv->dirtree));

	  break;
    }

    dirview_update_toolbar (dv);

    return FALSE;
}

static void
cb_dirview_select_file (GtkWidget * widget, gchar * path, DirView * dv)
{
    if (dv->lock_select)
	return;

    dirview_update_toolbar (dv);
    browser_select_dir (path);
}

static void
cb_dirview_refresh_dir_tree (GtkWidget * widget, DirView * dv)
{
    GtkCTreeNode *parent;

    browser->last_path = g_string_assign (browser->last_path, "");

    dirtree_freeze (DIRTREE (dirview->dirtree));

    parent = GTK_CLIST (dirview->dirtree)->selection->data;

    dirtree_refresh_tree ((DirTree *) dirview->dirtree, parent, TRUE);

    dirtree_thaw (DIRTREE (dirview->dirtree));

    dirview_update_toolbar (dirview);
}

static void
cb_dirview_expand (GtkWidget * widget, DirView * dv)
{
    GList  *sel_list;
    GtkCTreeNode *node;

    sel_list = GTK_CLIST (dv->dirtree)->selection;

    if (!sel_list)
	return;

    node = sel_list->data;

    dirtree_freeze (GTK_CTREE (dv->dirtree));
    gtk_ctree_expand (GTK_CTREE (dv->dirtree), node);
    dirtree_thaw (GTK_CTREE (dv->dirtree));

    dirview_update_toolbar (dirview);
}

static void
cb_dirview_collapse (GtkWidget * widget, DirView * dv)
{
    GList  *sel_list;
    GtkCTreeNode *node;

    sel_list = GTK_CLIST (dv->dirtree)->selection;
    if (!sel_list)
	return;

    node = sel_list->data;

    dirtree_freeze (GTK_CTREE (dv->dirtree));
    gtk_ctree_collapse (GTK_CTREE (dv->dirtree), node);
    dirtree_thaw (GTK_CTREE (dv->dirtree));

    dirview_update_toolbar (dirview);
}

static void
cb_dirview_down (GtkWidget * widget, DirView * dv)
{
    GtkCTreeNode *node;

    GList  *sel_list;

    sel_list = GTK_CLIST (dv->dirtree)->selection;

    if (!sel_list)
	return;

    node = sel_list->data;
    node = GTK_CTREE_NODE_NEXT (node);

    if (node != NULL)
    {
	dirtree_freeze (GTK_CTREE (dv->dirtree));

	gtk_ctree_select (GTK_CTREE (dv->dirtree), node);
	GTK_CLIST (dv->dirtree)->focus_row =
	    GTK_CLIST (dv->dirtree)->rows - g_list_length ((GList *) node);

	dirtree_thaw (GTK_CTREE (dv->dirtree));

	dirview_scroll_center ();
	dirview_update_toolbar (dirview);
    }
}

static void
cb_dirview_up (GtkWidget * widget, DirView * dv)
{
    GtkCTreeNode *node;

    GList  *sel_list;

    sel_list = GTK_CLIST (dv->dirtree)->selection;

    if (!sel_list)
	return;

    node = sel_list->data;

    node = GTK_CTREE_NODE_PREV (node);

    if (node != NULL)
    {
	dirtree_freeze (GTK_CTREE (dv->dirtree));

	gtk_ctree_select (GTK_CTREE (dv->dirtree), node);
	GTK_CLIST (dv->dirtree)->focus_row =
	    GTK_CLIST (dv->dirtree)->rows - g_list_length ((GList *) node);

	dirtree_thaw (GTK_CTREE (dv->dirtree));

	dirview_scroll_center ();
	dirview_update_toolbar (dirview);
    }
}

static void
cb_dirview_show_hidden (GtkWidget * widget, DirView * dv)
{
    dv->lock_select = TRUE;

    if (DIRTREE (dv->dirtree)->show_hidden)
	DIRTREE (dv->dirtree)->show_hidden = FALSE;
    else
	DIRTREE (dv->dirtree)->show_hidden = TRUE;

    cb_dirview_refresh_dir_tree (widget, dv);

    dv->lock_select = FALSE;
}

static GtkWidget *
dirview_create_toolbar (DirView * dv)
{
    GtkWidget *toolbar;
    GtkWidget *iconw;

    g_return_val_if_fail (dv, NULL);

#ifdef USE_GTK2
    toolbar = gtk_toolbar_new ();
#else
    toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
    gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), GTK_RELIEF_NONE);
    gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar),
				 GTK_TOOLBAR_SPACE_LINE);
    gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar), 16);
#endif

    /*
     * refresh 
     */
    iconw = pixbuf_create_pixmap_from_xpm_data (refresh_xpm);
    dv->toolbar_refresh_btn = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
						       "refresh",
						       "refresh",
						       "refresh",
						       iconw,
						       GTK_SIGNAL_FUNC
						       (cb_dirview_refresh_dir_tree),
						       dv);
    /*
     * up 
     */
    iconw = pixbuf_create_pixmap_from_xpm_data (up_xpm);
    dv->toolbar_up_btn = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
						  "up",
						  "up",
						  "up",
						  iconw,
						  GTK_SIGNAL_FUNC
						  (cb_dirview_up), dv);
    /*
     * down 
     */
    iconw = pixbuf_create_pixmap_from_xpm_data (down_xpm);
    dv->toolbar_down_btn = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
						    "down",
						    "down",
						    "down",
						    iconw,
						    GTK_SIGNAL_FUNC
						    (cb_dirview_down), dv);
    /*
     * collapse 
     */
    iconw = pixbuf_create_pixmap_from_xpm_data (left_xpm);
    dv->toolbar_collapse_btn = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
							"collapse",
							"collapse",
							"collapse",
							iconw,
							GTK_SIGNAL_FUNC
							(cb_dirview_collapse),
							dv);
    /*
     * expand 
     */
    iconw = pixbuf_create_pixmap_from_xpm_data (right_xpm);
    dv->toolbar_expand_btn = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
						      "expand",
						      "expand",
						      "expand",
						      iconw,
						      GTK_SIGNAL_FUNC
						      (cb_dirview_expand),
						      dv);

    /*
     * show/hide hidden directory
     */
    iconw = pixbuf_create_pixmap_from_xpm_data (dotfile_xpm);
    dv->toolbar_show_hidden_btn =
	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
				 "show/hide hidden directory",
				 "show/hide hidden directory",
				 "show/hide hidden directory", iconw,
				 GTK_SIGNAL_FUNC (cb_dirview_show_hidden),
				 dv);

    gtk_widget_show_all (toolbar);
    gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);

    return toolbar;
}

static void
dirview_update_toolbar (DirView * dv)
{
    GtkCTreeNode *node;
    GList  *sel_list;
    gboolean is_leaf, expanded;
    gint    current, len;

    sel_list = GTK_CLIST (dv->dirtree)->selection;

    if (!sel_list)
	return;

    node = sel_list->data;

    len = GTK_CLIST (GTK_CTREE (dv->dirtree))->rows;

    current = len - g_list_length ((GList *) node);

    if (current == 0)
    {
	gtk_widget_set_sensitive (dv->toolbar_up_btn, FALSE);
	gtk_widget_set_sensitive (dv->toolbar_down_btn, TRUE);
    }
    else if (current == (len - 1))
    {
	gtk_widget_set_sensitive (dv->toolbar_up_btn, TRUE);
	gtk_widget_set_sensitive (dv->toolbar_down_btn, FALSE);
    }
    else
    {
	gtk_widget_set_sensitive (dv->toolbar_up_btn, TRUE);
	gtk_widget_set_sensitive (dv->toolbar_down_btn, TRUE);
    }

    gtk_ctree_get_node_info (GTK_CTREE (dv->dirtree), node, NULL, NULL,
			     NULL, NULL, NULL, NULL, &is_leaf, &expanded);

    gtk_widget_set_sensitive (dv->toolbar_expand_btn, !is_leaf);
    gtk_widget_set_sensitive (dv->toolbar_collapse_btn, !is_leaf);

    if (is_leaf)
	return;

    if (expanded)
    {
	gtk_widget_set_sensitive (dv->toolbar_expand_btn, FALSE);
	gtk_widget_set_sensitive (dv->toolbar_collapse_btn, TRUE);
    }
    else
    {
	gtk_widget_set_sensitive (dv->toolbar_expand_btn, TRUE);
	gtk_widget_set_sensitive (dv->toolbar_collapse_btn, FALSE);
    }
}
