/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

static gint filelist_valcomp (gpointer a, gpointer b);
static void filelist_set_cursor (GtkWidget * widget,
				 GdkCursorType cursor_type);

FileList *filelist = NULL;

FileList *
filelist_init (void)
{
    FileList *fl;

    fl = g_new0 (FileList, 1);
    filelist_alloc_list (fl);

    return fl;
}

void
filelist_free (FileList * fl)
{

    if (fl->list != NULL)
	filelist_clear_list (fl);

    g_free (fl);
}

void
filelist_alloc_list (FileList * fl)
{
    FileListNode *data;

    fl->list = g_list_alloc ();
    data = g_malloc (sizeof (FileListNode));
    data->file_name = g_strdup ("");
    data->file_type = -1;

    fl->list->data = data;
}

void
filelist_add_to_list (FileList * fl, gchar * name, gint type)
{
    FileListNode *data;

    data = g_malloc (sizeof (FileListNode));
    data->file_name = g_strdup (name);
    data->file_type = type;

    fl->list =
	g_list_insert_sorted (fl->list, data,
			      (GCompareFunc) filelist_valcomp);

    fl->num++;
}

void
filelist_clear_list (FileList * fl)
{
    GList  *list_node;
    FileListNode *data;

    if (fl->list == NULL)
	return;

    for (list_node = fl->list; list_node; list_node = list_node->next)
    {
	data = list_node->data;

	if (data)
	{
	    if (data->file_name)
		g_free (data->file_name);

	    g_free (data);
	}

	list_node->data = NULL;
    }

    g_list_free (fl->list);

    fl->list = NULL;
    fl->num = 0;
}

void
filelist_create_list (FileList * fl, gchar * path)
{
    DIR    *dir;
    struct dirent *file;
    GtkWidget *top = gtk_widget_get_toplevel (GTK_WIDGET (browser->window));

    if (fl->list != NULL)
	filelist_clear_list (fl);

    if (path == NULL)
	return;

    dir = opendir (path);

    if (!dir)
	return;

    filelist_set_cursor (top, GDK_WATCH);

    filelist_alloc_list (fl);

    while ((file = readdir (dir)))
    {
	gint    file_type;

	if ((file->d_name[0] == '.' && file->d_name[1] == 0)
	    || (file->d_name[0] == '.' && file->d_name[1] == '.'
		&& file->d_name[2] == 0))
	    continue;

	file_type = image_type_check (file->d_name);

	if (isfile (file->d_name) && file_type != IMAGETYPE_UNKNOWN)
	    filelist_add_to_list (fl, file->d_name, file_type);

	while (gtk_events_pending ())
	    gtk_main_iteration ();
    }

    closedir (dir);
    filelist_set_cursor (top, -1);
}

static  gint
filelist_valcomp (gpointer a, gpointer b)
{
    FileListNode *node1, *node2;

    node1 = (FileListNode *) a;
    node2 = (FileListNode *) b;

    return strcmp ((gchar *) node1->file_name, (gchar *) node2->file_name);
}

static void
filelist_set_cursor (GtkWidget * widget, GdkCursorType cursor_type)
{
    GdkCursor *cursor = NULL;

    if (!widget || !widget->window)
	return;

    if (cursor_type > -1)
	cursor = gdk_cursor_new (cursor_type);

    gdk_window_set_cursor (widget->window, cursor);

    if (cursor)
	gdk_cursor_destroy (cursor);

    gdk_flush ();
}
