/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

gchar  *
home_dir (void)
{
    gchar  *home = getenv ("HOME");

    if (home)
	return home;
    else
    {
	struct passwd *pw = getpwuid (getuid ());

	if (pw)
	    return pw->pw_dir;
	else
	    return NULL;
    }
}

gint
isfile (const gchar * path)
{
    struct stat st;

    if ((!path) || (!*path))
	return FALSE;
    if (stat (path, &st) < 0)
	return FALSE;
    if (S_ISREG (st.st_mode))
	return TRUE;
    return FALSE;
}

gint
isdir (const gchar * path)
{
    struct stat st;

    if ((!path) || (!*path))
	return FALSE;
    if (stat (path, &st) < 0)
	return FALSE;
    if (S_ISDIR (st.st_mode))
	return TRUE;
    return FALSE;
}

gint
islink (const gchar * path)
{
    struct stat st;

    if (lstat (path, &st) < 0)
	return FALSE;
    if (S_ISLNK (st.st_mode))
	return TRUE;
    return FALSE;
}

gint
iswritable (const gchar * path)
{
    gint    uid = getuid ();
    gint    gid = getgid ();
    struct stat st;

    if (stat (path, &st))
	return FALSE;

    if (!S_ISDIR (st.st_mode))
	return FALSE;

    if (uid == st.st_uid && (st.st_mode & S_IWUSR))
	return TRUE;
    else if (gid == st.st_gid && (st.st_mode & S_IWGRP))
	return TRUE;
    else if (st.st_mode & S_IWOTH)
	return TRUE;

    return FALSE;
}

gint
makedir (const gchar * dir)
{
    if ((!dir) || (!*dir))
	return FALSE;

    if (!mkdir (dir, S_IRWXU))
	return TRUE;

    return FALSE;
}

time_t
filetime (const gchar * path)
{
    struct stat st;

    if ((!path) || (!*path))
	return 0;

    if (stat (path, &st) < 0)
	return 0;

    return st.st_mtime;
}

gint
filesize (const gchar * path)
{
    struct stat st;

    if ((!path) || (!*path))
	return 0;

    if (stat (path, &st) < 0)
	return 0;

    return (int) st.st_size;
}

gchar  *
extension (const gchar * path)
{
    gchar  *p = strrchr (path, '.');

    if (p == NULL)
	return "";
    else
	return p + 1;
}

const gchar *
filename_from_path (const gchar * path)
{
    if (!path)
	return NULL;

    return g_basename (path);
}

gchar  *
remove_level_from_path (const gchar * path)
{
    gchar  *new_path;
    const gchar *ptr = path;
    gint    p;

    if (!path)
	return NULL;

    p = strlen (path) - 1;

    if (p < 0)
	return NULL;

    while (ptr[p] != '/' && p > 0)
	p--;

    if (p == 0 && ptr[p] == '/')
	p++;

    new_path = g_strndup (path, (guint) p);

    return new_path;
}
