/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

#include "pixmaps/nav_button.xpm"

ImageView *imageview = NULL;

static gint width_backup;
static gint height_backup;

static GdkPixmap *buffer = NULL;
static gboolean move_scrollbar_by_user = TRUE;

/* callback functions */
static void cb_scrollbar_value_changed (GtkAdjustment * adj, ImageView * iv);
static gboolean cb_nav_button (GtkWidget * widget, GdkEventButton * event,
			       ImageView * iv);
static void cb_imageview_load_done (ImageLoader * il, gpointer data);
static void cb_imageview_load_error (ImageLoader * il, gpointer data);

static gboolean cb_image_configure (GtkWidget * widget,
				    GdkEventConfigure * event,
				    ImageView * iv);
static gboolean cb_image_expose (GtkWidget * widget, GdkEventExpose * event,
				 ImageView * iv);
static gboolean cb_image_motion_notify (GtkWidget * widget,
					GdkEventMotion * event,
					ImageView * iv);
static gboolean cb_image_button_press (GtkWidget * widget,
				       GdkEventButton * event,
				       ImageView * iv);
static gboolean cb_image_button_release (GtkWidget * widget,
					 GdkEventButton * event,
					 ImageView * iv);
static void cb_draw_area_map (GtkWidget * widget, ImageView * iv);
static gboolean cb_image_key_press (GtkWidget * widget, GdkEventKey * event,
				    ImageView * iv);
static gboolean cb_fullscreen_motion_notify (GtkWidget * widget,
					     GdkEventButton * bevent,
					     gpointer data);
static gint cb_slideshow (gpointer data);

/* local functions */
static void reset_scrollbar (ImageView * iv);
static void imageview_draw_image (ImageView * iv);
static void allocate_draw_buffer (ImageView * iv);
static void imageview_adjust_pos_in_frame (ImageView * iv, gboolean center);
static void imageview_open_navwin (ImageView * iv, gfloat x_root,
				   gfloat y_root);
static void imageview_calc_image_size (ImageView * iv);
static void imageview_calc_image (ImageView * iv);
static void imageview_clear_image (void);
static gboolean imageview_get_image_position (ImageView * iv, gint * x,
					      gint * y);
static void show_cursor (ImageView * iv);
static void hide_cursor (ImageView * iv);
static gint timeout_hide_cursor (gpointer data);
static void imageview_set_bg_color (ImageView * iv, gint red, gint green,
				    gint brue);
static void imageview_fullscreen_mode_unset (ImageView * iv);
static void imageview_slideshow_stop (ImageView * iv);

void
imageview_create (void)
{
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    GdkPixbuf *pixbuf;

    imageview = g_new0 (ImageView, 1);
    imageview->image = NULL;
    imageview->loader = NULL;
    imageview->pixmap = NULL;
    imageview->mask = NULL;
    imageview->width = imageview->height = 0;
    imageview->x_pos = imageview->y_pos = 0;
    imageview->x_scale = imageview->y_scale = 100;
    imageview->fit_to_frame = conf.image_fit;
    imageview->is_fullscreen = FALSE;
    imageview->is_slideshow = FALSE;
    imageview->is_hide = FALSE;

    if (imageview->fit_to_frame)
	imageview->zoom = IMAGEVIEW_ZOOM_FIT;

    /*
     * container 
     */
    imageview->container = gtk_frame_new (NULL);
    gtk_frame_set_shadow_type (GTK_FRAME (imageview->container),
			       GTK_SHADOW_IN);
    gtk_widget_show (imageview->container);

    /*
     * create widgets 
     */
    imageview->table = gtk_table_new (2, 2, FALSE);
    gtk_widget_show (imageview->table);
    gtk_container_add (GTK_CONTAINER (imageview->container),
		       imageview->table);

    imageview->draw_area = gtk_drawing_area_new ();
    gtk_widget_show (imageview->draw_area);

    imageview->hadj =
	GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 0.0, 10.0, 10.0, 0.0));
    imageview->hscrollbar = gtk_hscrollbar_new (imageview->hadj);
    gtk_widget_show (imageview->hscrollbar);

    imageview->vadj =
	GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 0.0, 10.0, 10.0, 0.0));
    imageview->vscrollbar = gtk_vscrollbar_new (imageview->vadj);
    gtk_widget_show (imageview->vscrollbar);

    imageview->event_box = gtk_event_box_new ();
    gtk_widget_set_name (imageview->event_box, "/imageview/NavWinButton");
    gtk_widget_show (imageview->event_box);

    pixbuf = gdk_pixbuf_new_from_xpm_data (nav_button_xpm);

    gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &mask, 127);

    imageview->nav_button = gtk_pixmap_new (pixmap, mask);

    gdk_pixbuf_unref (pixbuf);

    gtk_container_add (GTK_CONTAINER (imageview->event_box),
		       imageview->nav_button);
    gtk_widget_show (imageview->nav_button);

    gtk_table_attach (GTK_TABLE (imageview->table),
		      GTK_WIDGET (imageview->draw_area), 0, 1, 0, 1,
		      GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
    gtk_table_attach (GTK_TABLE (imageview->table), imageview->vscrollbar, 1,
		      2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (imageview->table), imageview->hscrollbar, 0,
		      1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
    gtk_table_attach (GTK_TABLE (imageview->table), imageview->event_box, 1,
		      2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);

    gtk_signal_connect_after (GTK_OBJECT (imageview->draw_area), "map",
			      GTK_SIGNAL_FUNC (cb_draw_area_map), imageview);
    gtk_signal_connect_after (GTK_OBJECT (imageview->draw_area),
			      "key-press-event",
			      GTK_SIGNAL_FUNC (cb_image_key_press),
			      imageview);

    gtk_signal_connect (GTK_OBJECT (imageview->draw_area), "configure_event",
			GTK_SIGNAL_FUNC (cb_image_configure), imageview);
    gtk_signal_connect (GTK_OBJECT (imageview->draw_area), "expose_event",
			GTK_SIGNAL_FUNC (cb_image_expose), imageview);
    gtk_signal_connect (GTK_OBJECT (imageview->draw_area),
			"motion_notify_event",
			GTK_SIGNAL_FUNC (cb_image_motion_notify), imageview);

    gtk_signal_connect (GTK_OBJECT (imageview->draw_area),
			"button_press_event",
			GTK_SIGNAL_FUNC (cb_image_button_press), imageview);

    gtk_signal_connect (GTK_OBJECT (imageview->draw_area),
			"button_release_event",
			GTK_SIGNAL_FUNC (cb_image_button_release), imageview);

    /*
     * set flags 
     */
    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (imageview->draw_area), GTK_CAN_FOCUS);

    gtk_widget_add_events (GTK_WIDGET (imageview->draw_area),
			   GDK_FOCUS_CHANGE
			   | GDK_BUTTON_PRESS_MASK | GDK_2BUTTON_PRESS
			   | GDK_KEY_PRESS | GDK_KEY_RELEASE
			   | GDK_BUTTON_RELEASE_MASK
			   | GDK_POINTER_MOTION_MASK
			   | GDK_POINTER_MOTION_HINT_MASK);

    /*
     * set signals 
     */
    gtk_signal_connect (GTK_OBJECT (imageview->hadj), "value_changed",
			GTK_SIGNAL_FUNC (cb_scrollbar_value_changed),
			imageview);

    gtk_signal_connect (GTK_OBJECT (imageview->vadj), "value_changed",
			GTK_SIGNAL_FUNC (cb_scrollbar_value_changed),
			imageview);

    gtk_signal_connect (GTK_OBJECT (imageview->event_box),
			"button_press_event", GTK_SIGNAL_FUNC (cb_nav_button),
			imageview);
}

void
imageview_destroy (void)
{
    if (buffer)
	gdk_pixmap_unref (buffer);

    if (imageview->loader)
	image_loader_free (imageview->loader);
    imageview->loader = NULL;

    if (imageview->pixmap)
	gdk_pixmap_unref (imageview->pixmap);

    if (imageview->mask)
	gdk_bitmap_unref (imageview->mask);

    g_free (imageview);
    imageview = NULL;
}

void
imageview_stop (void)
{
    if (imageview->is_fullscreen)
	imageview_fullscreen_mode_unset (imageview);

    if (imageview->loader)
	imageview_set_image (NULL);
}

void
imageview_set_image (gchar * path)
{
    imageview->keep_aspect = TRUE;

    image_loader_free (imageview->loader);
    imageview->loader = NULL;
    imageview->image = NULL;

    if (path == NULL)
    {
	imageview_clear_image ();
	return;
    }

    imageview->loader = image_loader_new (path);
    image_loader_set_error_func (imageview->loader, cb_imageview_load_error,
				 imageview);
    if (!image_loader_start
	(imageview->loader, cb_imageview_load_done, imageview))
    {
	image_loader_free (imageview->loader);
	imageview->loader = NULL;
	imageview->image = NULL;
	imageview_clear_image ();
    }
}

void
imageview_get_image_frame_size (ImageView * iv, gint * width, gint * height)
{
    g_return_if_fail (width && height);

    *width = *height = 0;

    g_return_if_fail (iv);

/* FIXME !!!!! */
    if (iv->draw_area->allocation.width == 1
	&& iv->draw_area->allocation.height == 1 && iv->is_fullscreen)
    {
	*width = width_backup;
	*height = height_backup;
    }
    else
    {
	*width = iv->draw_area->allocation.width;
	*height = iv->draw_area->allocation.height;
    }
}

void
imageview_zoom_image (ImageView * iv, ImgZoomType zoom,
		      gfloat x_scale, gfloat y_scale)
{
    gint    cx_pos, cy_pos, fwidth, fheight;
    gfloat  src_x_scale, src_y_scale;

    g_return_if_fail (iv);

    src_x_scale = iv->x_scale;
    src_y_scale = iv->y_scale;

    switch (zoom)
    {
      case IMAGEVIEW_ZOOM_IN:
	  if (iv->x_scale < IMG_MAX_SCALE && iv->y_scale < IMG_MAX_SCALE)
	  {
	      iv->x_scale = iv->x_scale + IMG_MIN_SCALE;
	      iv->y_scale = iv->y_scale + IMG_MIN_SCALE;
	  }
	  break;
      case IMAGEVIEW_ZOOM_OUT:
	  if (iv->x_scale > IMG_MIN_SCALE && iv->y_scale > IMG_MIN_SCALE)
	  {
	      iv->x_scale = iv->x_scale - IMG_MIN_SCALE;
	      iv->y_scale = iv->y_scale - IMG_MIN_SCALE;
	  }
	  break;
      case IMAGEVIEW_ZOOM_FIT:
	  iv->fit_to_frame = TRUE;
	  break;
      case IMAGEVIEW_ZOOM_100:
	  iv->x_scale = iv->y_scale = 100;
	  break;
	  break;
      default:
	  break;
    }

    imageview_get_image_frame_size (iv, &fwidth, &fheight);

    cx_pos = (iv->x_pos - fwidth / 2) * iv->x_scale / src_x_scale;
    cy_pos = (iv->y_pos - fheight / 2) * iv->y_scale / src_y_scale;

    iv->x_pos = cx_pos + fwidth / 2;
    iv->y_pos = cy_pos + fheight / 2;

    if (zoom != IMAGEVIEW_ZOOM_FIT)
	iv->fit_to_frame = FALSE;

    iv->zoom = zoom;

    imageview_calc_image (iv);
}

void
imageview_moveto (ImageView * iv, gint x, gint y)
{
    g_return_if_fail (iv);

    iv->x_pos = 0 - x;
    iv->y_pos = 0 - y;

    imageview_adjust_pos_in_frame (iv, TRUE);

    imageview_draw_image (iv);
    reset_scrollbar (iv);
}

void
imageview_fullscreen_mode_set (ImageView * iv)
{
    g_return_if_fail (iv);

    imageview_get_image_frame_size (iv, &width_backup, &height_backup);

    iv->is_fullscreen = TRUE;

    /*
     * create window
     */
    iv->fullscreen = gtk_window_new (GTK_WINDOW_POPUP);
    gtk_window_set_modal (GTK_WINDOW (iv->fullscreen), TRUE);
    gtk_window_set_wmclass (GTK_WINDOW (iv->fullscreen), "",
			    "pornview_fullscreen");
    gtk_widget_set_uposition (iv->fullscreen, 0, 0);
    gtk_widget_set_usize (iv->fullscreen,
			  gdk_screen_width (), gdk_screen_height ());

    gtk_signal_connect (GTK_OBJECT (iv->fullscreen),
			"motion_notify_event",
			(GtkSignalFunc) cb_fullscreen_motion_notify, iv);

    /*
     * set draw widget
     */
    imageview_set_bg_color (iv, 0, 0, 0);

    if (iv->x_pos < 0 || iv->y_pos)
	imageview_moveto (iv, 0, 0);

    gtk_widget_reparent (GTK_WIDGET (iv->draw_area), iv->fullscreen);

    gtk_widget_show (iv->fullscreen);

    gdk_keyboard_grab (iv->fullscreen->window, TRUE, GDK_CURRENT_TIME);
    gtk_grab_add (iv->fullscreen);
    gtk_widget_grab_focus (GTK_WIDGET (iv->draw_area));

    /*
     * enable auto hide cursor stuff
     */
    iv->hide_cursor_timer_id
	= gtk_timeout_add (IMGWIN_FULLSCREEN_HIDE_CURSOR_DELAY,
			   timeout_hide_cursor, iv);
}

void
imageview_slideshow_start (ImageView * iv)
{
    if (iv->is_fullscreen == FALSE)
	imageview_fullscreen_mode_set (iv);

    iv->slideshow_timer_id = gtk_timeout_add (conf.slideshow_interval,
					      (GtkFunction) cb_slideshow,
					      NULL);
    iv->is_slideshow = TRUE;
}

 /*
  * callback functions
  */

static void
cb_scrollbar_value_changed (GtkAdjustment * adj, ImageView * iv)
{
    g_return_if_fail (iv);

    if (!move_scrollbar_by_user)
	return;

    if (iv->width > iv->draw_area->allocation.width)
	iv->x_pos = 0 - iv->hadj->value;
    if (iv->height > iv->draw_area->allocation.height)
	iv->y_pos = 0 - iv->vadj->value;

    imageview_draw_image (iv);
}

static  gboolean
cb_nav_button (GtkWidget * widget, GdkEventButton * event, ImageView * iv)
{
    g_return_val_if_fail (iv, FALSE);

    imageview_open_navwin (iv, event->x_root, event->y_root);

    return FALSE;
}

static void
cb_imageview_load_done (ImageLoader * il, gpointer data)
{
    ImageView *iv = data;

    imageview->image = image_loader_get_pixbuf (il);

    if (imageview->image)
	imageview_calc_image (iv);
}

static void
cb_imageview_load_error (ImageLoader * il, gpointer data)
{
    image_loader_free (imageview->loader);
    imageview->loader = NULL;
    imageview->image = NULL;
    imageview_clear_image ();
}

static  gboolean
cb_image_configure (GtkWidget * widget, GdkEventConfigure * event,
		    ImageView * iv)
{
    gint    fwidth, fheight;

    imageview_get_image_frame_size (iv, &fwidth, &fheight);

    if (fwidth < iv->width)
    {
	if (iv->x_pos > 0 || iv->x_pos > fwidth || iv->x_pos < 0 - iv->width)
	    iv->x_pos = 0;
    }
    else
    {
	iv->x_pos = (fwidth - iv->width) / 2;
    }

    if (fheight < iv->height)
    {
	if (iv->y_pos > 0 || iv->y_pos > fheight
	    || iv->y_pos < 0 - iv->height)
	    iv->y_pos = 0;
    }
    else
    {
	iv->y_pos = (fheight - iv->height) / 2;
    }

    if (iv->x_pos < 0 || iv->y_pos < 0)
	imageview_moveto (iv, 0, 0);

    if (iv->zoom == IMAGEVIEW_ZOOM_FIT && iv->pixmap)
    {
	gint    width, height;

	gdk_window_get_size (iv->pixmap, &width, &height);

	if (fwidth != width || fheight != height)
	{
	    imageview_calc_image (iv);
	    reset_scrollbar (iv);

	    return TRUE;
	}
    }

    imageview_draw_image (iv);
    reset_scrollbar (iv);

    return TRUE;
}

static  gboolean
cb_image_expose (GtkWidget * widget, GdkEventExpose * event, ImageView * iv)
{
    if (iv->pixmap)
    {
	imageview_draw_image (iv);
    }

    return TRUE;
}

static  gboolean
cb_image_motion_notify (GtkWidget * widget, GdkEventMotion * event,
			ImageView * iv)
{
    GdkModifierType mods;
    gint    x, y, x_pos, y_pos, dx, dy;

    if (!iv->pressed)
	return FALSE;

    if (event->is_hint)
	gdk_window_get_pointer (widget->window, &x, &y, &mods);

    x_pos = x - iv->drag_startx;
    y_pos = y - iv->drag_starty;
    dx = x_pos - iv->x_pos_drag_start;
    dy = y_pos - iv->y_pos_drag_start;

    if (!iv->dragging && (abs (dx) > 2 || abs (dy) > 2))
	iv->dragging = TRUE;

    /*
     * scroll image 
     */
    if (iv->button == 1)
    {
	if (event->is_hint)
	    gdk_window_get_pointer (widget->window, &x, &y, &mods);

	iv->x_pos = x_pos;
	iv->y_pos = y_pos;

	imageview_adjust_pos_in_frame (iv, FALSE);
    }

    imageview_draw_image (iv);
    reset_scrollbar (iv);

    return TRUE;
}

static  gboolean
cb_image_button_press (GtkWidget * widget, GdkEventButton * event,
		       ImageView * iv)
{
    GdkCursor *cursor;
    gint    retval;

    g_return_val_if_fail (iv, FALSE);

    iv->pressed = TRUE;
    iv->button = event->button;

    gtk_widget_grab_focus (widget);

    if (iv->dragging)
	return FALSE;

    if (event->button == 1)
    {				/* scroll image */
	if (!iv->pixmap)
	    return FALSE;

	cursor = cursor_get (widget->window, CURSOR_HAND_CLOSED);

	retval = gdk_pointer_grab (widget->window, FALSE,
				   (GDK_POINTER_MOTION_MASK
				    | GDK_POINTER_MOTION_HINT_MASK
				    | GDK_BUTTON_RELEASE_MASK),
				   NULL, cursor, event->time);

	gdk_cursor_destroy (cursor);

	if (retval != 0)
	    return FALSE;

	iv->drag_startx = event->x - iv->x_pos;
	iv->drag_starty = event->y - iv->y_pos;
	iv->x_pos_drag_start = iv->x_pos;
	iv->y_pos_drag_start = iv->y_pos;

	return TRUE;

    }
    else if (event->button == 2 && iv->is_fullscreen)
    {
	imageview_fullscreen_mode_unset (iv);
	return TRUE;
    }

    return FALSE;
}

static  gboolean
cb_image_button_release (GtkWidget * widget, GdkEventButton * event,
			 ImageView * iv)
{
    gint    retval = TRUE;

    if (iv->pressed && !iv->dragging)
    {
	switch (event->button)
	{
	  case 1:
	  case 5:
	      thumbview_select_next ();
	      break;

	  case 3:
	  case 4:
	      thumbview_select_prev ();
	      break;
	}
    }
    iv->button = 0;
    iv->pressed = FALSE;
    iv->dragging = FALSE;

    gdk_pointer_ungrab (event->time);

    return retval;
}

static void
cb_draw_area_map (GtkWidget * widget, ImageView * iv)
{
    GdkCursor *cursor;

    imageview_set_bg_color (iv, 0, 0, 0);

    cursor = cursor_get (iv->draw_area->window, CURSOR_HAND_OPEN);
    gdk_window_set_cursor (iv->draw_area->window, cursor);
}

static  gboolean
cb_image_key_press (GtkWidget * widget, GdkEventKey * event, ImageView * iv)
{
    guint   keyval;
    GdkModifierType modval;
    gboolean move = FALSE;
    gint    mx, my;

    g_return_val_if_fail (iv, FALSE);

    if (iv->is_hide)
	return FALSE;

    imageview_get_image_position (iv, &mx, &my);

    keyval = event->keyval;
    modval = event->state;

    switch (keyval)
    {
      case GDK_F:
      case GDK_f:
	  if (iv->zoom != IMAGEVIEW_ZOOM_FIT)
	      imageview_zoom_image (iv, IMAGEVIEW_ZOOM_FIT, 0, 0);
	  return TRUE;

      case GDK_O:
      case GDK_o:
	  if (iv->zoom != IMAGEVIEW_ZOOM_100)
	      imageview_zoom_image (iv, IMAGEVIEW_ZOOM_100, 0, 0);
	  return TRUE;

      case GDK_plus:
      case GDK_KP_Add:
	  imageview_zoom_image (iv, IMAGEVIEW_ZOOM_IN, 0, 0);
	  return TRUE;

      case GDK_minus:
      case GDK_KP_Subtract:
	  imageview_zoom_image (iv, IMAGEVIEW_ZOOM_OUT, 0, 0);
	  return TRUE;

      case GDK_Left:
	  mx -= 20;
	  move = TRUE;
	  break;

      case GDK_Right:
	  mx += 20;
	  move = TRUE;
	  break;

      case GDK_Up:
	  my -= 20;
	  move = TRUE;
	  break;

      case GDK_Down:
	  my += 20;
	  move = TRUE;
	  break;

      case GDK_Page_Up:
      case GDK_P:
      case GDK_p:
	  thumbview_select_prev ();
	  return TRUE;

      case GDK_Page_Down:
      case GDK_N:
      case GDK_n:
	  thumbview_select_next ();
	  return TRUE;

      case GDK_Home:
	  thumbview_select_first ();
	  return TRUE;

      case GDK_End:
	  thumbview_select_last ();
	  return TRUE;

      case GDK_Return:
	  if (iv->is_fullscreen == FALSE)
	      imageview_fullscreen_mode_set (iv);
	  return TRUE;

      case GDK_S:
      case GDK_s:
	  if (iv->is_fullscreen)
	  {
	      if (iv->is_slideshow)
		  imageview_slideshow_stop (iv);
	      else
		  imageview_slideshow_start (iv);
	  }
	  return TRUE;

      case GDK_Q:
      case GDK_q:
      case GDK_Escape:
	  if (iv->is_fullscreen)
	      imageview_fullscreen_mode_unset (iv);
	  return TRUE;
    }

    if (move)
    {
	imageview_moveto (iv, mx, my);
	return TRUE;
    }

    return FALSE;
}

static  gboolean
cb_fullscreen_motion_notify (GtkWidget * widget,
			     GdkEventButton * bevent, gpointer data)
{
    ImageView *iv = data;

    gdk_window_get_pointer (widget->window, NULL, NULL, NULL);
    show_cursor (iv);

    if (iv->hide_cursor_timer_id != 0)
	gtk_timeout_remove (iv->hide_cursor_timer_id);
    iv->hide_cursor_timer_id
	= gtk_timeout_add (IMGWIN_FULLSCREEN_HIDE_CURSOR_DELAY,
			   timeout_hide_cursor, iv);

    return FALSE;
}

static  gint
cb_slideshow (gpointer data)
{
    gdk_threads_enter ();

    thumbview_select_next ();

    gdk_threads_leave ();

    return TRUE;
}

 /*
  * loacl functions 
  */

static void
reset_scrollbar (ImageView * iv)
{
    g_return_if_fail (iv);
    g_return_if_fail (iv->draw_area);
    g_return_if_fail (iv->hadj);
    g_return_if_fail (iv->vadj);

    /*
     * horizontal 
     */
    if (iv->x_pos < 0)
	iv->hadj->value = 0 - iv->x_pos;
    else
	iv->hadj->value = 0;

    if (iv->width > iv->draw_area->allocation.width)
	iv->hadj->upper = iv->width;
    else
	iv->hadj->upper = iv->draw_area->allocation.width;

    iv->hadj->page_size = iv->draw_area->allocation.width;

    /*
     * vertical 
     */
    if (iv->y_pos < 0)
	iv->vadj->value = 0 - iv->y_pos;
    else
	iv->vadj->value = 0;

    if (iv->height > iv->draw_area->allocation.height)
	iv->vadj->upper = iv->height;
    else
	iv->vadj->upper = iv->draw_area->allocation.height;

    iv->vadj->page_size = iv->draw_area->allocation.height;

    move_scrollbar_by_user = FALSE;

    gtk_signal_emit_by_name (GTK_OBJECT (iv->hadj), "changed");
    gtk_signal_emit_by_name (GTK_OBJECT (iv->vadj), "changed");

    move_scrollbar_by_user = TRUE;
}

static void
imageview_draw_image (ImageView * iv)
{
    GdkGC  *bg_gc;

    allocate_draw_buffer (iv);

    /*
     * fill background by default bg color 
     */
    bg_gc = iv->draw_area->style->black_gc;
    gdk_draw_rectangle (buffer, bg_gc, TRUE, 0, 0, -1, -1);

    /*
     * draw image to buffer 
     */

    if (iv->pixmap)
    {
	if (iv->mask)
	{
	    gdk_gc_set_clip_mask (iv->draw_area->style->black_gc, iv->mask);
	    gdk_gc_set_clip_origin (iv->draw_area->style->black_gc,
				    iv->x_pos, iv->y_pos);
	}

	gdk_draw_pixmap (buffer,
			 iv->draw_area->style->black_gc,
			 iv->pixmap, 0, 0, iv->x_pos, iv->y_pos, -1, -1);

	if (iv->mask)
	{
	    gdk_gc_set_clip_mask (iv->draw_area->style->black_gc, NULL);
	    gdk_gc_set_clip_origin (iv->draw_area->style->black_gc, 0, 0);
	}
    }

    /*
     * draw from buffer to foreground 
     */
    gdk_draw_pixmap (iv->draw_area->window,
		     iv->draw_area->style->
		     fg_gc[GTK_WIDGET_STATE (iv->draw_area)], buffer, 0, 0, 0,
		     0, -1, -1);
}

static void
allocate_draw_buffer (ImageView * iv)
{
    gint    fwidth, fheight;
    gint    cwidth, cheight;

    if (!iv)
	return;

    if (buffer)
    {
	gdk_window_get_size (buffer, &cwidth, &cheight);
	imageview_get_image_frame_size (iv, &fwidth, &fheight);

	if (fwidth != cwidth || fheight != cheight)
	{
	    gdk_pixmap_unref (buffer);
	    buffer = NULL;
	    buffer =
		gdk_pixmap_new (iv->draw_area->window, fwidth, fheight, -1);
	}
    }
    else
    {
	imageview_get_image_frame_size (iv, &fwidth, &fheight);
	buffer = gdk_pixmap_new (iv->draw_area->window, fwidth, fheight, -1);
    }
}


static gboolean imgview_scroll_nolimit = FALSE;

static void
imageview_adjust_pos_in_frame (ImageView * iv, gboolean center)
{
    gint    fwidth, fheight;

    if (center)
    {
	imageview_get_image_frame_size (iv, &fwidth, &fheight);

	if (fwidth < iv->width)
	{
	    if (imgview_scroll_nolimit)
	    {
		if (iv->x_pos > fwidth)
		    iv->x_pos = 0;
		else if (iv->x_pos < 0 - iv->width)
		    iv->x_pos = 0 - iv->width + fwidth;
	    }
	    else
	    {
		if (iv->x_pos > 0)
		    iv->x_pos = 0;
		else if (iv->x_pos < 0 - iv->width + fwidth)
		    iv->x_pos = 0 - iv->width + fwidth;
	    }
	}
	else
	{
	    iv->x_pos = (fwidth - iv->width) / 2;
	}

	if (fheight < iv->height)
	{
	    if (imgview_scroll_nolimit)
	    {
		if (iv->y_pos > fheight)
		    iv->y_pos = 0;
		else if (iv->y_pos < 0 - iv->height)
		    iv->y_pos = 0 - iv->height + fheight;
	    }
	    else
	    {
		if (iv->y_pos > 0)
		    iv->y_pos = 0;
		else if (iv->y_pos < 0 - iv->height + fheight)
		    iv->y_pos = 0 - iv->height + fheight;
	    }
	}
	else
	{
	    iv->y_pos = (fheight - iv->height) / 2;
	}
    }
    else
    {
	if (!imgview_scroll_nolimit)
	{
	    imageview_get_image_frame_size (iv, &fwidth, &fheight);

	    if (iv->width <= fwidth)
	    {
		if (iv->x_pos < 0)
		    iv->x_pos = 0;
		if (iv->x_pos > fwidth - iv->width)
		    iv->x_pos = fwidth - iv->width;
	    }
	    else if (iv->x_pos < fwidth - iv->width)
	    {
		iv->x_pos = fwidth - iv->width;
	    }
	    else if (iv->x_pos > 0)
	    {
		iv->x_pos = 0;
	    }

	    if (iv->height <= fheight)
	    {
		if (iv->y_pos < 0)
		    iv->y_pos = 0;
		if (iv->y_pos > fheight - iv->height)
		    iv->y_pos = fheight - iv->height;
	    }
	    else if (iv->y_pos < fheight - iv->height)
	    {
		iv->y_pos = fheight - iv->height;
	    }
	    else if (iv->y_pos > 0)
	    {
		iv->y_pos = 0;
	    }
	}
    }
}

static void
imageview_open_navwin (ImageView * iv, gfloat x_root, gfloat y_root)
{
    GdkPixbuf *image;
    GdkPixbuf *scale;

    GdkPixmap *pixmap;
    GdkBitmap *mask;
    gint    src_width, src_height, dest_width, dest_height;
    GdkColormap *cmap;

    g_return_if_fail (iv);
    if (!iv->pixmap)
	return;

    /*
     * get pixmap for navigator 
     */
    gdk_window_get_size (iv->pixmap, &src_width, &src_height);

    cmap = gdk_colormap_get_system ();
    image = gdk_pixbuf_get_from_drawable (NULL, iv->pixmap, cmap,
					  0, 0, 0, 0, src_width, src_height);

    g_return_if_fail (image);

    if (src_width > src_height)
    {
	dest_width = NAV_WIN_SIZE;
	dest_height = src_height * NAV_WIN_SIZE / src_width;
    }
    else
    {
	dest_height = NAV_WIN_SIZE;
	dest_width = src_width * NAV_WIN_SIZE / src_height;
    }

    scale =
	gdk_pixbuf_scale_simple (image, dest_width, dest_height,
				 conf.thumbnail_quality);
    gdk_pixbuf_render_pixmap_and_mask (scale, &pixmap, &mask, 64);

    if (!pixmap)
	goto ERROR;

    /*
     * open navigate window 
     */
    navwin_create (iv, x_root, y_root, pixmap, mask);

    /*
     * free 
     */
    gdk_pixmap_unref (pixmap);

  ERROR:
    gdk_pixbuf_unref (image);
}

static void
imageview_calc_image_size (ImageView * iv)
{
    gint    width, height;
    gfloat  x_scale, y_scale;

    if (iv->image == NULL)
	return;

    /*
     * image scale 
     */
    x_scale = iv->y_scale;
    y_scale = iv->x_scale;

    /*
     * calculate image size 
     */
    if (iv->fit_to_frame)
    {
	imageview_get_image_frame_size (iv, &width, &height);
	iv->x_scale = width / (gfloat) gdk_pixbuf_get_width (iv->image) * 100;
	iv->y_scale =
	    height / (gfloat) gdk_pixbuf_get_height (iv->image) * 100;


	if (iv->keep_aspect)
	{
	    if (iv->x_scale > iv->y_scale)
	    {
		iv->x_scale = iv->y_scale;
		width = gdk_pixbuf_get_width (iv->image) * iv->x_scale / 100;
	    }
	    else
	    {
		iv->y_scale = iv->x_scale;
		height =
		    gdk_pixbuf_get_height (iv->image) * iv->y_scale / 100;
	    }
	}
    }
    else
    {
	width = gdk_pixbuf_get_width (iv->image) * x_scale / 100;
	height = gdk_pixbuf_get_height (iv->image) * y_scale / 100;
    }

    if (width > 8)
	iv->width = width;
    else
	iv->width = 8;
    if (height > 8)
	iv->height = height;
    else
	iv->height = 8;
}

static void
imageview_calc_image (ImageView * iv)
{
    if (iv->image == NULL)
	return;

    iv->width = gdk_pixbuf_get_width (imageview->image);
    iv->height = gdk_pixbuf_get_height (imageview->image);

    if (imageview->pixmap)
	gdk_pixmap_unref (imageview->pixmap);

    if (imageview->mask)
	gdk_bitmap_unref (imageview->mask);

    imageview->pixmap = NULL;
    imageview->mask = NULL;

    if (iv->zoom != IMAGEVIEW_ZOOM_100)
    {
	GdkPixbuf *scale;

	imageview_calc_image_size (iv);

	scale =
	    gdk_pixbuf_scale_simple (imageview->image, iv->width, iv->height,
				     conf.image_quality);

	pixbuf_render_pixmap_and_mask (scale, &(iv->pixmap), &(iv->mask),
				       conf.image_dithering);

	gdk_pixbuf_unref (scale);
    }
    else
	pixbuf_render_pixmap_and_mask (imageview->image, &(iv->pixmap),
				       &(iv->mask), conf.image_dithering);

    imageview_adjust_pos_in_frame (iv, TRUE);

    imageview_draw_image (iv);
    reset_scrollbar (iv);
}

static void
imageview_clear_image (void)
{
    GdkGC  *bg_gc;

    if (imageview->loader)
	image_loader_free (imageview->loader);
    imageview->loader = NULL;

    imageview->image = NULL;

    if (imageview->pixmap)
	gdk_pixmap_unref (imageview->pixmap);

    if (imageview->mask)
	gdk_bitmap_unref (imageview->mask);


    imageview->image = NULL;
    imageview->pixmap = NULL;
    imageview->mask = NULL;

    imageview->width = 0;
    imageview->height = 0;
    imageview->x_pos = 0;
    imageview->y_pos = 0;

    bg_gc = imageview->draw_area->style->black_gc;
    gdk_draw_rectangle (imageview->draw_area->window, bg_gc, TRUE, 0, 0, -1,
			-1);

    reset_scrollbar (imageview);
}

static  gboolean
imageview_get_image_position (ImageView * iv, gint * x, gint * y)
{
    g_return_val_if_fail (iv, FALSE);
    g_return_val_if_fail (x && y, FALSE);

    *x = 0 - iv->x_pos;
    *y = 0 - iv->y_pos;

    return TRUE;
}

static void
show_cursor (ImageView * iv)
{
    GdkCursor *cursor;

    g_return_if_fail (iv);
    g_return_if_fail (iv->draw_area);
    g_return_if_fail (GTK_WIDGET_MAPPED (iv->draw_area));

    cursor = cursor_get (iv->draw_area->window, CURSOR_HAND_OPEN);
    gdk_window_set_cursor (iv->draw_area->window, cursor);
    gdk_cursor_destroy (cursor);
}

static void
hide_cursor (ImageView * iv)
{
    GdkCursor *cursor;

    g_return_if_fail (iv);
    g_return_if_fail (iv->draw_area);
    g_return_if_fail (GTK_WIDGET_MAPPED (iv->draw_area));

    cursor = cursor_get (iv->draw_area->window, CURSOR_VOID);
    gdk_window_set_cursor (iv->draw_area->window, cursor);
    gdk_cursor_destroy (cursor);
}

static  gint
timeout_hide_cursor (gpointer data)
{
    ImageView *iv = data;

    hide_cursor (iv);
    iv->hide_cursor_timer_id = 0;

    return FALSE;
}

static void
imageview_set_bg_color (ImageView * iv, gint red, gint green, gint brue)
{
    GdkColor *bg_color;

    g_return_if_fail (iv);
    g_return_if_fail (iv->draw_area);

    bg_color = g_new0 (GdkColor, 1);
    bg_color->pixel = 0;

    bg_color->red = red;
    bg_color->green = green;
    bg_color->blue = brue;

    if (GTK_WIDGET_MAPPED (iv->draw_area))
    {
	GdkColormap *colormap;
	GtkStyle *style;
	colormap = gdk_window_get_colormap (iv->draw_area->window);
	gdk_colormap_alloc_color (colormap, bg_color, FALSE, TRUE);
	style = gtk_style_copy (gtk_widget_get_style (iv->draw_area));
	style->bg[GTK_STATE_NORMAL] = *bg_color;
	gtk_widget_set_style (iv->draw_area, style);
	gtk_style_unref (style);
    }
}

void
imageview_fullscreen_mode_unset (ImageView * iv)
{
    g_return_if_fail (iv);

    if (iv->is_slideshow)
	imageview_slideshow_stop (iv);

    /*
     * restore draw widget 
     */

    gtk_widget_reparent (GTK_WIDGET (iv->draw_area), iv->table);

    gdk_keyboard_ungrab (GDK_CURRENT_TIME);
    gtk_widget_grab_focus (GTK_WIDGET (iv->draw_area));

    /*
     * disable auto hide cursor stuff 
     */
    if (iv->hide_cursor_timer_id != 0)
	gtk_timeout_remove (iv->hide_cursor_timer_id);
    show_cursor (iv);

    gtk_widget_destroy (iv->fullscreen);

    iv->fullscreen = NULL;
    iv->is_fullscreen = FALSE;
}

static void
imageview_slideshow_stop (ImageView * iv)
{
    if (iv->slideshow_timer_id)
    {
	gtk_timeout_remove (iv->slideshow_timer_id);
	iv->slideshow_timer_id = 0;
    }

    iv->is_slideshow = FALSE;
}
