/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

#define BUF_SIZE 4096
#define MAX_PATH_LEN 1024

Config  conf;

typedef enum
{
    D_NULL,
    D_STRING,
    D_PATH,
    D_INT,
    D_FLOAT,
    D_BOOL,
    D_ENUM
}
DataType;

typedef struct _ConfParm
{
    gchar  *keyname;
    DataType data_type;
    gchar  *default_val;
    gpointer data;
}
ConfParam;

typedef struct _PrefsSection
{
    gchar  *section_name;
    ConfParam *param;
}
PrefsSection;


typedef struct _PrefsFile
{
    gchar  *filename;
    PrefsSection *sections;
}
PrefsFile;

static void prefs_load_config_default (PrefsSection * sections);
static void prefs_load_rc (gchar * filename, PrefsSection * sections);
static void prefs_save_rc (gchar * filename, PrefsSection * sections);
static void store_config (gpointer data, gchar * val, DataType type);

/* default config value */

static ConfParam param_general[] = {
    {"startup_dir_mode", D_INT, "0", &conf.startup_dir_mode},
    {"startup_dire", D_STRING, NULL, &conf.startup_dir},
    {"save_win_state", D_BOOL, "TRUE", &conf.save_win_state},
    {"window_width", D_INT, "800", &conf.window_width},
    {"window_height", D_INT, "600", &conf.window_height},
    {"dirtree_width", D_INT, "360", &conf.dirtree_width},
    {"dirtree_height", D_INT, "207", &conf.dirtree_height},
    {NULL, D_NULL, NULL, NULL}
};

static ConfParam param_dirview[] = {
    {"scan_dir", D_BOOL, "TRUE", &conf.scan_dir},
    {"check_hlinks", D_BOOL, "TRUE", &conf.check_hlinks},
    {"show_hidden", D_BOOL, "FALSE", &conf.show_hidden},
    {"dirtree_line_style", D_INT, "2", &conf.dirtree_line_style},
    {"dirtree_expander_style", D_INT, "1", &conf.dirtree_expander_style},
    {NULL, D_NULL, NULL, NULL}
};


static ConfParam param_thumbview[] = {
    {"thumbnail_quality", D_INT, "1", &conf.thumbnail_quality},
    {"enable_thumb_dirs", D_BOOL, "FALSE", &conf.enable_thumb_dirs},
    {"enable_thumb_caching", D_BOOL, "TRUE", &conf.enable_thumb_caching},
    {"thumbview_mode", D_INT, "1", &conf.thumbview_mode},
    {NULL, D_NULL, NULL, NULL}
};

static ConfParam param_imageview[] = {
    {"image_quality", D_INT, "1", &conf.image_quality},
    {"image_dithering", D_INT, "1", &conf.image_dithering},
    {"image_fit", D_BOOL, "FALSE", &conf.image_fit},
    {NULL, D_NULL, NULL, NULL}
};

static ConfParam param_tools[] = {
    {"slideshow_interval", D_INT, "4000", &conf.slideshow_interval},
    {NULL, D_NULL, NULL, NULL}
};

static PrefsSection conf_sections[] = {
    {"General", param_general},
    {"Directory View", param_dirview},
    {"Image View", param_imageview},
    {"Thumbnail View", param_thumbview},
    {"Tools", param_tools},
    {NULL, NULL},
};

static PrefsFile conf_files[] = {
    {PORNVIEW_RC_FILE, conf_sections},
    {NULL, NULL},
};

void
prefs_load_config (void)
{
    gint    i;

    for (i = 0; conf_files[i].filename; i++)
	prefs_load_rc (conf_files[i].filename, conf_files[i].sections);
}

void
prefs_save_config (void)
{
    gint    i;

    for (i = 0; conf_files[i].filename; i++)
	prefs_save_rc (conf_files[i].filename, conf_files[i].sections);

    if (conf.startup_dir)
	g_free (conf.startup_dir);
}

static void
prefs_load_config_default (PrefsSection * sections)
{
    gint    i, j;

    for (j = 0; sections[j].section_name; j++)
    {
	ConfParam *param = sections[j].param;
	for (i = 0; param[i].keyname; i++)
	    store_config (param[i].data, param[i].default_val,
			  param[i].data_type);
    }

    if (conf.startup_dir_mode == 0)
	conf.startup_dir = g_strdup (getenv ("HOME"));

    return;
}

static void
prefs_load_rc (gchar * filename, PrefsSection * sections)
{
    gchar   rcfile[MAX_PATH_LEN];
    FILE   *pornviewrc;
    gchar   buf[BUF_SIZE];
    gchar **pair;
    gint    i, j;

    prefs_load_config_default (sections);

    g_snprintf (rcfile, MAX_PATH_LEN, "%s/%s/%s",
		getenv ("HOME"), PORNVIEW_RC_DIR, filename);

    pornviewrc = fopen (rcfile, "r");

    if (!pornviewrc)
    {
	return;
    }

    while (fgets (buf, sizeof (buf), pornviewrc))
    {
	if (buf[0] == '[' || buf[0] == '\n')
	    continue;

	g_strstrip (buf);

	pair = g_strsplit (buf, "=", 2);
	if (pair[0])
	    g_strstrip (pair[0]);
	if (pair[1])
	    g_strstrip (pair[1]);

	for (j = 0; sections[j].section_name; j++)
	{
	    ConfParam *param = sections[j].param;
	    for (i = 0; param[i].keyname; i++)
	    {
		if (!g_strcasecmp (param[i].keyname, pair[0]))
		{
		    store_config (param[i].data, pair[1], param[i].data_type);
		    break;
		}
	    }
	}
	g_strfreev (pair);
    }

    fclose (pornviewrc);
}

static void
prefs_save_rc (gchar * filename, PrefsSection * sections)
{
    gchar   dir[MAX_PATH_LEN], rcfile[MAX_PATH_LEN];
    FILE   *pornviewrc;
    gint    i, j;
    gchar  *bool;
    struct stat st;

    g_snprintf (dir, MAX_PATH_LEN, "%s/%s", getenv ("HOME"), PORNVIEW_RC_DIR);

    if (stat (dir, &st))
    {
	mkdir (dir, S_IRWXU);
    }
    else
    {
	if (!S_ISDIR (st.st_mode))
	{
	    return;
	}
    }

    g_snprintf (rcfile, MAX_PATH_LEN, "%s/%s/%s", getenv ("HOME"),
		PORNVIEW_RC_DIR, filename);
    pornviewrc = fopen (rcfile, "w");
    if (!pornviewrc)
    {
	return;
    }

    for (j = 0; sections[j].section_name; j++)
    {
	ConfParam *param = sections[j].param;
	fprintf (pornviewrc, "[%s]\n", sections[j].section_name);
	for (i = 0; param[i].keyname; i++)
	{
	    switch (param[i].data_type)
	    {
	      case D_STRING:
		  if (!*(gchar **) param[i].data)
		      fprintf (pornviewrc, "%s=\n", param[i].keyname);
		  else
		      fprintf (pornviewrc, "%s=%s\n", param[i].keyname,
			       *(gchar **) param[i].data);
		  break;
	      case D_INT:
		  fprintf (pornviewrc, "%s=%d\n", param[i].keyname,
			   *((gint *) param[i].data));
		  break;
	      case D_FLOAT:
		  fprintf (pornviewrc, "%s=%f\n", param[i].keyname,
			   *((gfloat *) param[i].data));
		  break;
	      case D_BOOL:
		  if (*((gboolean *) param[i].data))
		      bool = "TRUE";
		  else
		      bool = "FALSE";
		  fprintf (pornviewrc, "%s=%s\n", param[i].keyname, bool);
		  break;
	      case D_ENUM:
		  fprintf (pornviewrc, "%s=%d\n", param[i].keyname,
			   *((gint *) param[i].data));
		  break;
	      default:
		  break;
	    }
	}
	fprintf (pornviewrc, "\n");
    }

    fclose (pornviewrc);
}

static void
store_config (gpointer data, gchar * val, DataType type)
{
    switch (type)
    {
      case D_STRING:
	  g_free (*((gchar **) data));
	  if (!val)
	      *((gchar **) data) = NULL;
	  else
	      *((gchar **) data) = g_strdup (val);
	  break;
      case D_INT:
	  if (!val)
	      break;
	  *((gint *) data) = atoi (val);
	  break;
      case D_FLOAT:
	  if (!val)
	      break;
	  *((gfloat *) data) = atof (val);
	  break;
      case D_BOOL:
	  if (!val)
	      break;
	  if (!g_strcasecmp (val, "TRUE"))
	      *((gboolean *) data) = TRUE;
	  else
	      *((gboolean *) data) = FALSE;
	  break;
      case D_ENUM:
	  if (!val)
	      break;
	  *((gint *) data) = atoi (val);
	  break;
      default:
	  break;
    }
}
