/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

GtkWidget *slideshow_interval_label = NULL;

static const gint intervals[] = {
    1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000,
    15000, 20000, 30000, 45000, 60000, 2 * 60000, 3 * 60000, 4 * 60000,
    5 * 60000, 6 * 60000, 7 * 60000, 8 * 60000, 9 * 60000, 10 * 60000,
	20 * 60000,
    30 * 60000, 40 * 60000, 60 * 60000, 2 * 60 * 60000, 4 * 60 * 60000,
	8 * 60 * 60000,
    24 * 60 * 60000
};

static const gint nintervals = 32;
static guint s_interval;

static gint slideshow_interval_get_index (gint interval);
static void slideshow_interval_update (gint interval);

static void cb_slideshow_interval_scale_changed (GtkAdjustment * adjustment,
						 gpointer data);

GtkWidget *
prefs_tools_page (void)
{
    GtkWidget *main_vbox;

    main_vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 10);

    return main_vbox;
}

GtkWidget *
prefs_slideshow_page (void)
{
    GtkWidget *main_vbox;
    GtkWidget *frame1;
    GtkWidget *vbox;
    GtkWidget *frame2;
    GtkObject *adjustment;
    GtkWidget *hscale;

    main_vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 10);

    frame1 = gtk_frame_new (" Delay before image change: ");
    gtk_box_pack_start (GTK_BOX (main_vbox), frame1, FALSE, FALSE, 0);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
    gtk_container_add (GTK_CONTAINER (frame1), vbox);

    frame2 = gtk_frame_new (NULL);
    gtk_box_pack_start (GTK_BOX (vbox), frame2, FALSE, TRUE, 1);
    gtk_widget_set_usize (frame2, -2, 30);

    adjustment = gtk_adjustment_new (5.0, 0.0, (gfloat) nintervals,
				     1.0, 1.0, 1.0);

    gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
			GTK_SIGNAL_FUNC (cb_slideshow_interval_scale_changed),
			NULL);

    GTK_ADJUSTMENT (adjustment)->value =
	(gfloat) slideshow_interval_get_index (conf_temp->slideshow_interval);

    hscale = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
    gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
    gtk_scale_set_digits (GTK_SCALE (hscale), 1);
    gtk_container_add (GTK_CONTAINER (frame2), hscale);
    gtk_scale_set_draw_value (GTK_SCALE (hscale), FALSE);

    slideshow_interval_label = gtk_label_new ("");
    gtk_box_pack_start (GTK_BOX (vbox), slideshow_interval_label, FALSE,
			FALSE, 3);

    slideshow_interval_update (conf_temp->slideshow_interval);

    return main_vbox;
}

static gint
slideshow_interval_get_index (gint interval)
{
    gint    i;

    for (i = 0; i < nintervals; i++)
    {
	if (interval <= intervals[i])
	    return i;
    }

    return nintervals - 1;
}

static void
slideshow_interval_update (gint interval)
{
    static const guchar *unit_name[] = {
	"second(s)",
	"minute(s)",
	"hour(s)",
	"day(s)"
    };
    static const gint unit_count[] = {
	1000, 60, 60, 24
    };
    gint    i;
    guchar  buf[80];

    s_interval = interval;
    if (interval == 0)
    {
	gtk_label_set (GTK_LABEL (slideshow_interval_label),
		       "milliseconds, no delay");
    }
    else
    {
	i = 0;
	interval = (interval + 999) / 1000;

	while (i < 3 && interval >= unit_count[i + 1])
	{
	    i++;
	    interval /= unit_count[i];
	}
	sprintf (buf, "%u milliseconds, %i %s", s_interval, interval,
		 unit_name[i]);

	gtk_label_set (GTK_LABEL (slideshow_interval_label), buf);
    }
}

static void
cb_slideshow_interval_scale_changed (GtkAdjustment * adjustment,
				     gpointer data)
{
    conf_temp->slideshow_interval = intervals[(gint) adjustment->value];
    slideshow_interval_update (conf_temp->slideshow_interval);
}
