/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "pornview.h"

#include "gedo-paned.h"
#include "gedo-hpaned.h"
#include "gedo-vpaned.h"

static GtkCTreeNode *navigate_tree_get_parent (PrefsPage * page);
static GtkWidget *create_navigate_tree_and_pages (void);


static void cb_ctree_select_row (GtkWidget * widget, gint row, gint column,
				 GdkEventButton * event, gpointer data);

static void cb_prefs_window_close (void);
static void cb_prefs_window_apply_button (void);
static void cb_prefs_window_cancel_button (void);

typedef struct PrefsWin_Tag
{
    GtkWidget *notebook;
    GtkWidget *ctree;
}
PrefsWin;

Config *conf_temp = NULL;

static void conf_to_temp (void);
static void conf_from_temp (void);
static void conf_temp_free (void);

static GtkWidget *prefs_window;
static PrefsWin prefs_win;

static PrefsPage prefs_pages[] = {
    {"General", NULL, NULL, prefs_general_page, NULL, NULL},
    {"Directory View", "General", NULL, prefs_dirview_page, NULL, NULL},
    {"Image View", "General", NULL, prefs_imageview_page, NULL, NULL},
    {"Thumbnail View", "General", NULL, prefs_thumbview_page, NULL, NULL},
    {"Tools", NULL, NULL, prefs_tools_page, NULL, NULL},
    {"Slideshow", "Tools", NULL, prefs_slideshow_page, NULL, NULL},
};

static gint prefs_pages_num = sizeof (prefs_pages) / sizeof (prefs_pages[0]);

void
prefs_window_open (void)
{
    GtkWidget *notebook;
    GtkWidget *pane;
    GtkWidget *scrolledwin;
    GtkWidget *button;

    /*
     * if preference window is alredy opend, raise it and return 
     */
    if (prefs_window)
    {
	gdk_window_raise (prefs_window->window);
	return;
    }

    conf_to_temp ();

    /*
     * create config window 
     */
    prefs_window = gtk_dialog_new ();

#ifndef USE_GTK2
    GTK_WINDOW (prefs_window)->type = GTK_WINDOW_DIALOG;
#endif

    gtk_window_set_position (GTK_WINDOW (prefs_window), GTK_WIN_POS_CENTER);
    gtk_window_set_wmclass (GTK_WINDOW (prefs_window), "prefs", "PornView");
    gtk_window_set_title (GTK_WINDOW (prefs_window), "Preference");
    gtk_window_set_default_size (GTK_WINDOW (prefs_window), 600, 340);

    gtk_window_set_transient_for (GTK_WINDOW (prefs_window),
				  GTK_WINDOW (browser->window));

    gtk_signal_connect (GTK_OBJECT (prefs_window), "destroy",
			GTK_SIGNAL_FUNC (cb_prefs_window_close), NULL);

    /*
     * pane 
     */
    pane = gedo_hpaned_new ();
    gtk_container_set_border_width (GTK_CONTAINER (pane), 5);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_window)->vbox),
			pane, TRUE, TRUE, 0);

    notebook = prefs_win.notebook = gtk_notebook_new ();
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
    gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
    gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook));
    gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);

    scrolledwin = create_navigate_tree_and_pages ();

    gedo_paned_add1 (GEDO_PANED (pane), scrolledwin);
    gedo_paned_add2 (GEDO_PANED (pane), notebook);

    /*
     * dialog buttons 
     */
    button = gtk_button_new_with_label ("Apply");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_window)->action_area),
			button, TRUE, TRUE, 0);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       GTK_SIGNAL_FUNC (cb_prefs_window_apply_button),
			       GTK_OBJECT (prefs_window));
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

    button = gtk_button_new_with_label ("Cancel");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs_window)->action_area),
			button, TRUE, TRUE, 0);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       GTK_SIGNAL_FUNC
			       (cb_prefs_window_cancel_button),
			       GTK_OBJECT (prefs_window));
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button);

    gtk_widget_show_all (prefs_window);
}

static GtkCTreeNode *
navigate_tree_get_parent (PrefsPage * page)
{
    gint    i;

    g_return_val_if_fail (page, NULL);
    g_return_val_if_fail (page->title, NULL);

    if (!page->parent)
	return NULL;

    for (i = 0; i < prefs_pages_num; i++)
    {
	if (!strcmp (page->parent, prefs_pages[i].title))
	{
	    return prefs_pages[i].node;
	}
    }

    return NULL;
}

static GtkWidget *
create_navigate_tree_and_pages (void)
{
    GtkWidget *scrolledwin, *ctree;
    GtkWidget *vbox = NULL, *label;
    GtkCTreeNode *node, *parent = NULL;
    GtkCTreeNode *sel_node = NULL;
    gchar  *text;
    gint    i;

    scrolledwin = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwin),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize (scrolledwin, 170, -1);

    ctree = prefs_win.ctree = gtk_ctree_new (1, 0);
    gtk_clist_set_column_auto_resize (GTK_CLIST (ctree), 0, TRUE);
    gtk_clist_set_selection_mode (GTK_CLIST (ctree), GTK_SELECTION_BROWSE);
    gtk_ctree_set_line_style (GTK_CTREE (ctree), GTK_CTREE_LINES_DOTTED);
    gtk_ctree_set_expander_style (GTK_CTREE (ctree),
				  GTK_CTREE_EXPANDER_SQUARE);
    gtk_clist_set_row_height (GTK_CLIST (ctree), 18);
    gtk_container_add (GTK_CONTAINER (scrolledwin), ctree);
    gtk_widget_show (ctree);

    gtk_signal_connect (GTK_OBJECT (ctree), "select-row",
			GTK_SIGNAL_FUNC (cb_ctree_select_row), NULL);

    for (i = 0; i < prefs_pages_num; i++)
    {
	GdkPixmap *pixmap = NULL, *opixmap = NULL;
	GdkBitmap *mask = NULL, *omask = NULL;

	if (!prefs_pages[i].title)
	    continue;

	/*
	 * translate page title 
	 */
	text = prefs_pages[i].title;

	/*
	 * create page 
	 */
	if (prefs_pages[i].create_page_fn)
	{
	    vbox = prefs_pages[i].create_page_fn ();
	    label = gtk_label_new (text);
	    gtk_notebook_append_page (GTK_NOTEBOOK (prefs_win.notebook),
				      vbox, label);
	}

	if (prefs_pages[i].parent)
	    parent = navigate_tree_get_parent (&prefs_pages[i]);

	/*
	 * insert node 
	 */
	node = gtk_ctree_insert_node (GTK_CTREE (ctree), parent, NULL,
				      &text, 4,
				      pixmap, mask,
				      opixmap, omask, FALSE, FALSE);
	if (parent)
	    gtk_ctree_expand (GTK_CTREE (ctree), parent);

	if (vbox)
	{
	    gtk_ctree_node_set_row_data_full (GTK_CTREE (ctree), node,
					      vbox, NULL);
	    prefs_pages[i].page = vbox;

	    if (i == 0)
		sel_node = node;

	}
	prefs_pages[i].node = node;

	vbox = NULL;
	parent = NULL;
    }

    gtk_ctree_select (GTK_CTREE (ctree), sel_node);

    return scrolledwin;
}


static void
cb_ctree_select_row (GtkWidget * widget, gint row, gint column,
		     GdkEventButton * event, gpointer data)
{
    GtkCTreeNode *node;
    GtkWidget *page;
    gint    num;

    /*
     * gtk_clist_get_selection_info (GTK_CLIST (widget),
     * event->x, event->y,
     * &row, NULL);
     */
    node = gtk_ctree_node_nth (GTK_CTREE (widget), row);
    if (!node)
	return;

    page = gtk_ctree_node_get_row_data (GTK_CTREE (widget), node);

    if (!page || !GTK_IS_WIDGET (page))
	return;

    num = gtk_notebook_page_num (GTK_NOTEBOOK (prefs_win.notebook), page);
    if (num >= 0)
	gtk_notebook_set_page (GTK_NOTEBOOK (prefs_win.notebook), num);
}

static void
cb_prefs_window_close ()
{
    gint    i;

    prefs_window = NULL;

    for (i = 0; i < prefs_pages_num; i++)
    {
	prefs_pages[i].node = NULL;
	prefs_pages[i].page = NULL;
    }
}

static void
cb_prefs_window_apply_button ()
{
    conf_from_temp ();

    gtk_widget_destroy (prefs_window);
}


static void
cb_prefs_window_cancel_button ()
{
    conf_temp_free ();

    gtk_widget_destroy (prefs_window);
}

static void
conf_to_temp (void)
{
    conf_temp = g_memdup (&conf, sizeof (Config));
    conf_temp->startup_dir = g_strdup (conf.startup_dir);
}

static void
conf_from_temp (void)
{
    gboolean changed = FALSE;

    if (conf.startup_dir_mode != conf_temp->startup_dir_mode)
    {
	conf.startup_dir_mode = conf_temp->startup_dir_mode;
	changed = TRUE;
    }

    if (changed)
    {
	if (conf.startup_dir_mode == 2)
	{
	    if (conf.startup_dir)
		g_free (conf.startup_dir);

	    conf.startup_dir = g_strdup (conf_temp->startup_dir);
	}
	else if (conf.startup_dir_mode == 0)
	{
	    if (conf.startup_dir)
		g_free (conf.startup_dir);

	    conf.startup_dir = g_strdup (home_dir ());
	}

	changed = FALSE;
    }

    if (conf.save_win_state != conf_temp->save_win_state)
	conf.save_win_state = conf_temp->save_win_state;

    if (conf.scan_dir != conf_temp->scan_dir)
    {
	conf.scan_dir = conf_temp->scan_dir;
	changed = TRUE;
    }

    if (conf.check_hlinks != conf_temp->check_hlinks)
    {
	conf.check_hlinks = conf_temp->check_hlinks;
	changed = TRUE;
    }

    if (conf.show_hidden != conf_temp->show_hidden)
    {
	conf.show_hidden = conf_temp->show_hidden;
	changed = TRUE;
    }

    if (conf.dirtree_line_style != conf_temp->dirtree_line_style)
    {
	conf.dirtree_line_style = conf_temp->dirtree_line_style;
	changed = TRUE;
    }

    if (conf.dirtree_expander_style != conf_temp->dirtree_expander_style)
    {
	conf.dirtree_expander_style = conf_temp->dirtree_expander_style;
	changed = TRUE;
    }

    if (changed)
    {
	dirtree_set_mode (DIRTREE (dirview->dirtree), conf.scan_dir,
			  conf.check_hlinks, conf.show_hidden,
			  conf.dirtree_line_style,
			  conf.dirtree_expander_style);
	changed = FALSE;
    }

    if (conf.image_quality != conf_temp->image_quality)
    {
	conf.image_quality = conf_temp->image_quality;
	changed = TRUE;
    }

    if (conf.image_dithering != conf_temp->image_dithering)
    {
	conf.image_dithering = conf_temp->image_dithering;
	changed = TRUE;
    }

    if (conf.image_fit != conf_temp->image_fit)
    {
	conf.image_fit = conf_temp->image_fit;
	changed = TRUE;
    }

    if (changed)
    {
	if (conf.image_fit)
	    imageview_zoom_image (imageview, IMAGEVIEW_ZOOM_FIT, 0, 0);
	else
	    imageview_zoom_image (imageview, IMAGEVIEW_ZOOM_100, 0, 0);

	changed = FALSE;
    }

    if (conf.thumbnail_quality != conf_temp->thumbnail_quality)
	conf.thumbnail_quality = conf_temp->thumbnail_quality;

    if (conf.enable_thumb_dirs != conf_temp->enable_thumb_dirs)
	conf.enable_thumb_dirs = conf_temp->enable_thumb_dirs;

    if (conf.enable_thumb_caching != conf_temp->enable_thumb_caching)
	conf.enable_thumb_caching = conf_temp->enable_thumb_caching;

    if (conf.thumbview_mode != conf_temp->thumbview_mode)
    {
	conf.thumbview_mode = conf_temp->thumbview_mode;
	changed = TRUE;
    }

    if (changed)
    {
	thumbview_set_mode (conf.thumbview_mode);
	changed = FALSE;
    }

    if (conf.slideshow_interval != conf_temp->slideshow_interval)
	conf.slideshow_interval = conf_temp->slideshow_interval;

    conf_temp_free ();
}

static void
conf_temp_free (void)
{
    if (conf_temp == NULL)
	return;
    if (conf_temp->startup_dir)
	g_free (conf_temp->startup_dir);
    g_free (conf_temp);
    conf_temp = NULL;
}
